@@ -12,7 +12,7 @@ use feature ();
 package Zydeco;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.521';
+our $VERSION   = '0.600';
 
 use Keyword::Simple ();
 use PPR;
@@ -1022,7 +1022,7 @@ my $_should_optimize = sub {
 	my ($code, $sigvars) = @_;
 	
 	my %allowed = ( '$self' => undef, '$class' => undef, '$_' => undef, '@_' => undef );
-	undef $allowed{$_} for split /\s*,\s*/, $sigvars;
+	undef $allowed{$_} for split /\s*,\s*/, $sigvars//'';
 	
 	my @vars = ( $code =~ /[\$\@\%]\w+/g );
 	foreach my $var (@vars) {
@@ -1154,7 +1154,7 @@ sub _handle_method_keyword {
 	}
 	
 	if ($lex_name) {
-		return "my $lex_name = $return";
+		return "my $lex_name = $return; &Internals::SvREADONLY(\\$lex_name, 1);";
 	}
 	
 	return $return;
@@ -1376,6 +1376,7 @@ sub _handle_has_keyword {
 	my @names = $me->_handle_name_list($names);
 	
 	my @r;
+	my @make_read_only;
 	for my $name (@names) {
 		$name =~ s/^\+\*/+/;
 		$name =~ s/^\*//;
@@ -1390,6 +1391,7 @@ sub _handle_has_keyword {
 				$name,
 				$rawspec,
 			);
+			push @make_read_only, $name;
 		}
 		else {
 			push @r, sprintf(
@@ -1400,6 +1402,15 @@ sub _handle_has_keyword {
 			);
 		}
 	}
+	
+	if (@make_read_only) {
+		push @r, sprintf(
+			'q[%s]->_end(sub { &Internals::SvREADONLY($_, 1) for %s; })',
+			$me,
+			join(q{,}, map("\\$_", @make_read_only)),
+		);
+	}
+	
 	join ";", @r;
 }
 
@@ -2339,7 +2350,7 @@ sub _include {
 #{
 #	package Zydeco::Anonymous::Package;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.521';
+#	our $VERSION   = '0.600';
 #	use overload q[""] => sub { ${$_[0]} }, fallback => 1;
 #	sub DESTROY {}
 #	sub AUTOLOAD {
@@ -2350,7 +2361,7 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Class;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.521';
+#	our $VERSION   = '0.600';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub new {
 #		my $me = shift;
@@ -2363,12 +2374,12 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Role;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.521';
+#	our $VERSION   = '0.600';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	
 #	package Zydeco::Anonymous::ParameterizableClass;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.521';
+#	our $VERSION   = '0.600';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -2382,7 +2393,7 @@ sub _include {
 #
 #	package Zydeco::Anonymous::ParameterizableRole;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.521';
+#	our $VERSION   = '0.600';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
